@@ -11,6 +11,9 @@ use JSON;
 use IO::Handle;
 use IO::File;
 use IPC::Run;
+use File::Basename qw(basename dirname);
+use File::Path qw(make_path);
+use File::Copy qw();
 use File::Spec qw(); ##-- for tmpdir()
 use File::Temp qw(); ##-- for tempdir(), tempfile()
 use Fcntl qw(:DEFAULT SEEK_SET SEEK_CUR SEEK_END);
@@ -31,13 +34,13 @@ our %EXPORT_TAGS =
      sort  => [qw(csort_to csortuc_to)],
      run   => [qw(crun opencmd)],
      env   => [qw(env_set env_push env_pop)],
-     pack  => [qw(packsize packFilterFetch packFilterStore)],
+     pack  => [qw(packsize packsingle packFilterFetch packFilterStore)],
      math  => [qw($LOG2 log2 min2 max2 lmax lmin lsum)],
      list  => [qw(luniq sluniq xluniq)],
      regex => [qw(regex)],
      html  => [qw(htmlesc)],
      time  => [qw(s2hms s2timestr timestamp)],
-     file  => [qw(file_mtime file_timestamp du_file du_glob)],
+     file  => [qw(file_mtime file_timestamp du_file du_glob copyto moveto copyto_a cp_a)],
      si    => [qw(si_str)],
      pdl   => [qw(_intersect_p _union_p _complement_p _setdiff_p),
 	       qw(readPdlFile writePdlFile writePdlHeader writeCcsHeader mmzeroes mmtemp),
@@ -321,9 +324,19 @@ sub csortuc_to {
 sub packsize {
   use bytes; #use bytes; ##-- deprecated in perl v5.18.2
   no warnings;
+  shift if (UNIVERSAL::isa($_[0],__PACKAGE__));
   return bytes::length(pack($_[0],@_[1..$#_]));
 }
 
+## $bool = PACKAGE::packsingle($packfmt)
+## $bool = PACKAGE::packsingle($packfmt,@args)
+##  + guess whether $packfmt is a single-element (scalar) format
+sub packsingle {
+  shift if (UNIVERSAL::isa($_[0],__PACKAGE__));
+  return (packsize($_[0],0)==packsize($_[0],0,0)
+	  && $_[0] !~ m{\*|(?:\[(?:[2-9]|[0-9]{2,})\])|(?:[[:alpha:]].*[[:alpha:]])});
+}
+
 ## \&filter_sub = PACKAGE::packFilterStore($pack_template)
 ## \&filter_sub = PACKAGE::packFilterStore([$pack_template_store, $pack_template_fetch])
 ## \&filter_sub = PACKAGE::packFilterStore([\&pack_code_store,   \&pack_code_fetch])
@@ -334,7 +347,7 @@ sub packFilterStore {
   $packas    = $packas->[0] if (UNIVERSAL::isa($packas,'ARRAY'));
   return $packas  if (UNIVERSAL::isa($packas,'CODE'));
   return undef    if (!$packas || $packas eq 'raw');
-  if (packsize($packas,0)==packsize($packas,0,0)) {
+  if ($that->packsingle($packas)) {
     return sub {
       $_ = pack($packas,$_) if (defined($_));
     };
@@ -355,7 +368,7 @@ sub packFilterFetch {
   $packas    = $packas->[1] if (UNIVERSAL::isa($packas,'ARRAY'));
   return $packas  if (UNIVERSAL::isa($packas,'CODE'));
   return undef    if (!$packas || $packas eq 'raw');
-  if (packsize($packas,0)==packsize($packas,0,0)) {
+  if ($that->packsingle($packas)) {
     return sub {
       $_ = unpack($packas,$_);
     };
@@ -553,6 +566,77 @@ sub du_glob {
   return du_file(map {glob($_)} @_);
 }
 
+## $bool = PACKAGE->copyto($srcfile,   $dstdir, %opts)
+## $bool = PACKAGE->copyto(\@srcfiles, $dstdir, %opts)
+##  + copies file(s) $srcfile (first form) or @srcfiles (second form) to $dstdir, creating $dstdir if it doesn't already exist;
+##    options %opts:
+##    (
+##     from   => $from,      ##-- replace prefix $from in file(s) with $todir; default=undef: flat copy to $todir
+##     method => \&method,   ##-- use CODE-ref \&method as underlying copy routing; default=\&File::Copy::copy
+##     label  => $label,     ##-- report errors as '$label'; (default='copyto()')
+##    )
+sub copyto {
+  my $that = UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+  my ($srcfiles,$todir,%opts) = @_;
+  my $method = $opts{method} || \&File::Copy::copy;
+  my $label  = $opts{label}  || 'copyto()';
+  my $from   = $opts{from};
+  my ($src,$dst,$dstdir);
+  foreach $src (UNIVERSAL::isa($srcfiles,'ARRAY') ? @$srcfiles : $srcfiles) {
+    if (defined($from)) {
+      ($dst = $src) =~ s{^\Q$from\E}{$todir};
+    } else {
+      $dst = "$todir/".basename($src);
+    }
+    $dstdir = dirname($dst);
+    -d $dstdir
+      or make_path($dstdir)
+      or $that->logconfess("$label: failed to create target directory '$dstdir': $!");
+    $method->($src,$dst)
+      or $that->logconfess("$label: failed to transfer file '$src' to to '$dst': $!");
+  }
+  return 1;
+}
+
+## $bool = PACKAGE->copyto_a($src,$dstdir,%opts)
+## + wrapper for PACKAGE->copyto($src,$dstdir, %opts,method=>PACKAGE->can('cp_a'),label=>'copyto_a()')
+sub copyto_a {
+  my $that = UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+  return $that->copyto(@_, method=>\&cp_a, label=>'copyto_a()');
+}
+
+## $bool = PACKAGE->moveto($src,$dstdir, %opts)
+## + wrapper for PACKAGE->copyto($src,$dstdir, %opts,method=>\&File::Copy::move,label=>'moveto()')
+sub moveto {
+  my $that = UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+  return $that->copyto(@_, method=>\&File::Copy::move, label=>'moveto()');
+}
+
+## $bool = PACKAGE->cp_a($src,$dst)
+## $bool = PACKAGE->cp_a($src,$dstdir)
+##  + copies file $src to $dst, propagating ownership, permissions, and timestamps
+sub cp_a {
+  my $that = UNIVERSAL::isa($_[0],__PACKAGE__) ? shift : __PACKAGE__;
+  my ($src,$dst) = @_;
+  if (File::Copy->can('syscopy') && File::Copy->can('syscopy') ne File::Copy->can('copy')) {
+    ##-- use File::copy::syscopy() if available
+    return File::Copy::syscopy($src,$dst,3);
+  }
+  ##-- copy and then manually propagate file attributes
+  my $rc = File::Copy::copy($src,$dst) or return undef;
+  $dst = "$dst/".basename($src) if (-d $dst);
+  my @stat = stat($src);
+  my ($perm,$gid,$atime,$mtime) = @stat[2,5,8,9];
+  my $uid = $>==0 ? $stat[4] : $>;  ##-- don't try to set uid unless we're running as root
+  $rc &&= CORE::chown($uid,$gid,$dst)
+      or $that->warn("cp_a(): failed to propagate ownership from '$src' to '$dst': $!");
+  $rc &&= CORE::chmod(($perm & 07777), $dst)
+      or $that->warn("cp_a(): failed to propagate persmissions from '$src' to '$dst': $!");
+  $rc &&= CORE::utime($atime,$mtime,$dst)
+      or $that->warn("cp_a(): failed to propagate timestamps from '$src' to '$dst': $!");
+  return $rc;
+}
+
 ##==============================================================================
 ## Utils: SI
 
@@ -779,8 +863,8 @@ sub mmtemp {
   return DiaColloDB::PDL::MM::mmtemp(@_);
 }
 
-## mmunlink(@mmfiles)
-## mmunlink($mmpdl,@mmfiles)
+## $bool = mmunlink(@mmfiles)
+## $bool = mmunlink($mmpdl,@mmfiles)
 ##  + unlinkes file(s) generated by mmzeroes($basename)
 sub mmunlink {
   require DiaColloDB::PDL::MM;
@@ -943,7 +1027,7 @@ sub tmparrayp {
   return \@tmparray;
 }
 
-## \%tmphash = CLASS->tmphash($class, $template, %opts)
+## \%tmphash = CLASS->tmphash($template, %opts)
 ##  + ties a new temporary hash via $class (default='DB_File')
 ##  + calls tie(my @tmparray, $class, $tmpfilename, temp=>1, %opts)
 sub tmphash {
